@@ -7,8 +7,8 @@ PyTweet is a Synchronous python API wrapper for Twitter's API!
 :copyright: (c) 2021 TheFarGG & TheGenocides
 :license: MIT, see LICENSE for more details.
 """
-from typing import Literal, NamedTuple
 import logging
+from typing import Literal, NamedTuple
 
 from .attachments import *
 from .auth import *
@@ -16,17 +16,20 @@ from .client import *
 from .entities import *
 from .enums import *
 from .errors import *
+from .expansions import *
 from .http import *
 from .message import *
 from .metrics import *
 from .relations import *
+from .space import *
+from .stream import *
 from .tweet import *
 from .user import *
 from .utils import *
 
 __title__ = "PyTweet"
-__version__ = "1.2.5"
-__authors__ = ["TheFarGG", "TheGenocides"]
+__version__ = "1.3.5"
+__author__ = "TheFarGG & TheGenocides"
 __license__ = "MIT"
 __copyright__ = "Copyright 2021 TheFarGG & TheGenocides"
 
@@ -39,6 +42,8 @@ class VersionInfo(NamedTuple):
     serial: int
 
 
-version_info: VersionInfo = VersionInfo(major=1, minor=2, micro=5, releaselevel="final", serial=0)
+version_info: VersionInfo = VersionInfo(major=1, minor=3, micro=5, releaselevel="final", serial=0)
 
 logging.getLogger(__name__).addHandler(logging.NullHandler())
+
+assert version_info.releaselevel in ("alpha", "beta", "candidate", "final")
